unit prefs;
{$H+}

interface
uses IniFiles,SysUtils,define_types,graphics,Dialogs,Classes;
const
  knMRU = 12;
  kNoRender = 0;
  kFastRender = 1;
  kNormalRender = 2;
  //kSuperRender = 3;
type
  TMRU =  array [1..knMRU] of string;
  TPrefs = record
         ProportionalStretch,OverlayColorFromZero,
         MaskOverlayWithBackground, InterpolateOverlays,Frame,Perspective,OrthoSliceView,
         ColorbarText,SliceText,CrossHairs,Colorbar,ForcePowerOfTwo,//LowResolutionRendering,
         Show2DSlicesDuringRendering,IntelWarning: boolean;

         FormWidth,FormHeight,RenderQuality,BackgroundAlpha,OverlayAlpha,CrosshairThick: integer;
         BackgroundColor,CLUTWindowColor,CLUTIntensityColor: TColor;
         TextColor,TextBorder,ColorBarBorder,CrosshairColor: TGLRGBQuad;
         ColorBarPos: TUnitRect;
         PrevFilename,PrevScriptName: TMRU;
  end;
function IniFile(lRead: boolean; lFilename: string; var lPrefs: TPrefs): boolean;
procedure Add2MRU (var lMRU: TMRU;  var lNewFilename: string); //add new file to most-recent list
procedure IniByte(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: byte);
procedure IniInt(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: integer);
procedure IniFloat(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: single);
procedure IniRGBA(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: TGLRGBQuad);
procedure SetDefaultPrefs (var lPrefs: TPrefs; lEverything: boolean);
procedure FillMRU (var lMRU: TMRU; lSearchPath,lSearchExt: string; lForce: boolean);

implementation

procedure FillMRU (var lMRU: TMRU; lSearchPath,lSearchExt: string; lForce: boolean);
//e.g. SearchPath  includes final pathdelim, e.g. c:\filedir\
var
	lSearchRec: TSearchRec;
  lI,lMax: integer;
  lS: TStringList;
begin

  if (not lForce) and (lMRU[1] <> '') then
      exit; //only fill empty MRUs...
  lS := TStringList.Create;
  if FindFirst(lSearchPath+'*'+lSearchExt, faAnyFile, lSearchRec) = 0 then
	 repeat
      lS.Add(lSearchPath+lSearchRec.Name) ;
      //showmessage(lSearchRec.Name);
		  //LUTdrop.Items.Add(ParseFileName(ExtractFileName(lSearchRec.Name)));
      //convertCLUT(CLUTdir+pathdelim+lSearchRec.Name);
	 until (FindNext(lSearchRec) <> 0);
  FindClose(lSearchRec);
  lMax := lS.count;

  if lMax > 0 then begin
    lS.sort;
    if lMax > knMRU then
      lMax := knMRU;
    for lI := 1 to lMax do
      lMRu[lI] := lS[lI-1];
  end;
  Freeandnil(lS);
end;//UpdateColorSchemes

procedure IniFloat(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: single);
//read or write an integer value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('FLT',lIdent,FloattoStr(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('FLT',lIdent, '');
	if length(lStr) > 0 then
		lValue := StrToFloat(lStr);
end; //IniFloat

procedure IniByte(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: byte);
//read or write an integer value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('BYT',lIdent,InttoStr(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('BYT',lIdent, '');
	if length(lStr) > 0 then
		lValue := StrToInt(lStr);
end; //IniFloat

procedure Add2MRU (var lMRU: TMRU;  var lNewFilename: string); //add new file to most-recent list
var
  lStr: string;
  lPos,lN : integer;
begin
  //first, increase position of all old MRUs
  lN := 0; //Number of MRU files
  for lPos := 1 to (knMRU) do begin//first, eliminate duplicates
	  lStr := lMRU[lPos];
	  if (lStr <> '') and (lStr <> lNewFileName) then begin
		 inc(lN);
		 lMRU[lN] := lStr;
	  end; //keep in MRU list
  end; //for each MRU
  //next, increment positions
  if lN >= knMRU then
	 lN := knMRU - 1;
  for lPos := lN downto 1 do
	  lMRU[lPos+1] := lMRU[lPos];
  if (lN+2) < (knMRU) then //+1 as we have added a file
	 for lPos := (lN+2) to knMRU do
	   lMRU[lPos] := '';
  lMRU[1] := lNewFilename;
end;//Add2MRU

procedure SetDefaultPrefs (var lPrefs: TPrefs; lEverything: boolean);
begin
  if lEverything then begin  //These values are typically not changed...
       with lPrefs do begin
            CrossHairs := true;
            TextColor := RGBA(255,255,255,128);
            ColorBarBorder := RGBA(92,92,132,168);
            TextBorder := RGBA(92,92,132,255);
            CrosshairColor := RGBA (92,92,132,168);
            CLUTIntensityColor := RGBA2TColor (RGBA (102,26,77,255) );
            CLUTWindowColor := RGBA2TColor (RGBA (92,92,132,255) );
       end;
  end;
  with lPrefs do begin
    ProportionalStretch := true;
    Frame := false;
    OrthoSliceView := false;
    ColorbarText := true;
    SliceText := false;
    ForcePowerOfTwo:= false;
    //LowResolutionRendering := false;
    IntelWarning := true;
    Show2DSlicesDuringRendering := true;
    ColorBar := true;
    ColorBarPos:= CreateUnitRect (0.1,0.1,0.9,0.14);
    SensibleUnitRect (ColorBarPos);
    Perspective := false;
    InterpolateOverlays := false;
    MaskOverlayWithBackground := true;
    OverlayColorFromZero := false;
    //SurfaceThreshold := 25;
    BackgroundAlpha := 50;
    OverlayAlpha := 50;
    BackgroundColor := 0;
    CrosshairThick := 1;
    RenderQuality := kNormalRender;
    FormWidth := 530;
    FormHeight := 566;
    //CLUTIntensityColor := RGBA2TColor (RGBA (18,80,18,255) );
  end;//with lPrefs
end; //Proc SetDefaultPrefs

procedure SetDefaultPrefsMRU (var lPrefs: TPrefs);
var
  lI: integer;
begin
    SetDefaultPrefs(lPrefs,true);
    for lI := 1 to knMRU do begin
      lPrefs.PrevFilename[lI] := '';
      lPrefs.PrevScriptName[lI] := '';
    end;
end;




procedure IniInt(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: integer);
//read or write an integer value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('INT',lIdent,IntToStr(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('INT',lIdent, '');
	if length(lStr) > 0 then
		lValue := StrToInt(lStr);
end; //IniInt

procedure IniBool(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: boolean);
//read or write a boolean value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('BOOL',lIdent,Bool2Char(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('BOOL',lIdent, '');
	if length(lStr) > 0 then
		lValue := Char2Bool(lStr[1]);
end; //IniBool

procedure IniStr(lRead: boolean; lIniFile: TIniFile; lIdent: string; var lValue: string);
//read or write a string value to the initialization file
begin
  if not lRead then begin
    lIniFile.WriteString('STR',lIdent,lValue);
    exit;
  end;
	lValue := lIniFile.ReadString('STR',lIdent, '');
end; //IniStr

procedure IniUnitRect(lRead: boolean; lIniFile: TIniFile; lIdent: string; var lValue: TUnitRect);
var
  lS: string;
  lU: TUnitRect;
begin
  if not lRead then begin
    lIniFile.WriteString('STR',lIdent,UnitRectToStr (lValue));
    exit;
  end;
	lS := lIniFile.ReadString('STR',lIdent, '');
  if StrToUnitRect (lS, lU) then
    lValue := lU;
end; //IniStr

procedure IniRGBA(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: TGLRGBQuad);
//read or write an integer value to the initialization file
var
	lStr: string;
begin
  if not lRead then begin
    //lI64 := lValue.rgbred + lValue.rgbGreen shl 8 + lValue.rgbBlue shl 16 + lValue.rgbReserved shl 24;
    //lIniFile.WriteString('RGBA',lIdent,InttoStr(lI64));
    lIniFile.WriteString('RGBA255',lIdent,RGBAToStr(lValue));
    exit;
  end;
	lStr := lIniFile.ReadString('RGBA255',lIdent, '');
  StrToRGBA(lStr,lValue);
end; //IniRGBA

procedure IniColor(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: TColor);
//read or write an color value to the initialization file
var
	lC: TGLRGBQuad;
begin
  TColor2RGBA(lValue,lC);
  IniRGBA(lRead,lIniFile,lIdent,lC);
  lValue := RGBA2TColor(lC);
end; //IniColor

procedure IniMRU(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lMRU: TMRU);
var
	lI,lOK: integer;
begin
  if lRead then begin //compress files so lowest values are OK
    lOK := 0;
    for lI := 1 to knMRU do begin
      IniStr(lRead,lIniFile,lIdent+inttostr(lI),lMRU[lI]);
	    if (length(lMRU[lI]) > 0) and (fileexistsex(lMRU[lI])) then begin
		    inc(lOK);
		    lMRU[lOK] := lMRU[lI];
      end;//OK
	  end; //for each MRU
  end else
	  for lI := 1 to knMRU do
      IniStr(lRead,lIniFile,lIdent+inttostr(lI),lMRU[lI]);
end;

function IniFile(lRead: boolean; lFilename: string; var lPrefs: TPrefs): boolean;
//Read or write initialization variables to disk
var
  lIniFile: TIniFile;
  //lI: integer;
begin
//showmessage(lFilename);
  result := false;
  if (lRead) then
    SetDefaultPrefsMRU (lPrefs);
  if (lRead) and (not Fileexists(lFilename)) then begin
        //FillEmptyMRU(lPrefs);
        exit;
  end;
  lIniFile := TIniFile.Create(lFilename);
	IniBool(lRead,lIniFile, 'Frame',lPrefs.Frame);
	IniBool(lRead,lIniFile, 'ProportionalStretch',lPrefs.ProportionalStretch);
	//IniBool(lRead,lIniFile, 'ForceOrthogonal',lPrefs.ForceOrthogonal);
	IniBool(lRead,lIniFile, 'OrthoSliceView',lPrefs.OrthoSliceView);
	IniBool(lRead,lIniFile, 'SliceText',lPrefs.SliceText);
	IniBool(lRead,lIniFile, 'ColorBarText',lPrefs.ColorBarText);
	IniBool(lRead,lIniFile, 'CrossHairs',lPrefs.CrossHairs);
	IniBool(lRead,lIniFile, 'ForcePowerOfTwo',lPrefs.ForcePowerOfTwo);
	//IniBool(lRead,lIniFile, 'LowResolutionRendering',lPrefs.LowResolutionRendering);
	IniBool(lRead,lIniFile, 'IntelWarning',lPrefs.IntelWarning);
	IniBool(lRead,lIniFile, 'Show2DSlicesDuringRendering',lPrefs.Show2DSlicesDuringRendering);
	IniBool(lRead,lIniFile, 'ColorBar',lPrefs.ColorBar);
	IniBool(lRead,lIniFile, 'Perspective',lPrefs.Perspective);
	IniBool(lRead,lIniFile, 'InterpolateOverlays',lPrefs.InterpolateOverlays);
	IniBool(lRead,lIniFile, 'OverlayColorFromZero',lPrefs.OverlayColorFromZero);
  //The MIP is unusual, so lets always turn it off when the users restarts the software
	//lPrefs.MaximumIntensityProjection := IniBool(lIniFile, 'MaximumIntensityProjection',lPrefs.MaximumIntensityProjection);
  //IniInt(lRead,lIniFile, 'SurfaceThreshold',lPrefs.SurfaceThreshold);
  IniInt(lRead,lIniFile, 'BackgroundAlpha',lPrefs.BackgroundAlpha);
  IniInt(lRead,lIniFile, 'OverlayAlpha',lPrefs.OverlayAlpha);
  IniInt(lRead,lIniFile, 'FormHeight',lPrefs.FormHeight);
  IniInt(lRead,lIniFile, 'FormWidth',lPrefs.FormWidth);
  IniInt(lRead,lIniFile, 'RenderQuality',lPrefs.RenderQuality);
  IniInt(lRead,lIniFile, 'CrosshairThick',lPrefs.CrosshairThick);
  lPrefs.RenderQuality := Bound(lPrefs.RenderQuality,0,kNormalRender);
  IniColor(lRead,lIniFile, 'BackgroundColor',lPrefs.BackgroundColor);
  IniColor(lRead,lIniFile, 'CLUTWindowColor',lPrefs.CLUTWindowColor);
  IniColor(lRead,lIniFile, 'CLUTIntensityColor',lPrefs.CLUTIntensityColor);
  IniUnitRect(lRead,lIniFile, 'ColorBarPos',lPrefs.ColorBarPos);
  SensibleUnitRect (lPrefs.ColorBarPos);
  IniRGBA(lRead,lIniFile, 'TextColor',lPrefs.TextColor);
  IniRGBA(lRead,lIniFile, 'CrosshairColor',lPrefs.CrosshairColor);
  IniRGBA(lRead,lIniFile, 'TextBorder',lPrefs.TextBorder);
  IniRGBA(lRead,lIniFile, 'ColorBarBorder',lPrefs.ColorBarBorder);
//  IniRGBA(lRead,lIniFile, 'TextColor'+inttostr(lI),lPrefs.TextColor);
//  IniRGBA(lRead,lIniFile, 'TextBorder'+inttostr(lI),lPrefs.TextBorder);
//  IniRGBA(lRead,lIniFile, 'ColorBarBorder'+inttostr(lI),lPrefs.ColorBarBorder);
  IniMRU(lRead,lIniFile,'PrevFilename',lPrefs.PrevFilename);
  IniMRU(lRead,lIniFile,'PrevScriptName',lPrefs.PrevScriptName);
  lIniFile.Free;
end;

end.
